@@ -506,6 +506,9 @@ if mswindows:
                              STD_INPUT_HANDLE, STD_OUTPUT_HANDLE,
                              STD_ERROR_HANDLE, SW_HIDE,
                              STARTF_USESTDHANDLES, STARTF_USESHOWWINDOW)
+    # https://msdn.microsoft.com/en-us/library/windows/desktop/ms687032(v=vs.85).aspx
+    # Note: In Python 3.3 this constant is found in the _winapi module.
+    _WAIT_TIMEOUT = 0x102
 
     __all__.extend(["CREATE_NEW_CONSOLE", "CREATE_NEW_PROCESS_GROUP",
                     "STD_INPUT_HANDLE", "STD_OUTPUT_HANDLE",
@@ -1144,7 +1147,7 @@ class Popen(object):
                 timeout = int(timeout * 1000)
             if self.returncode is None:
                 result = _subprocess.WaitForSingleObject(self._handle, timeout)
-                if result == _subprocess.WAIT_TIMEOUT:
+                if result == _WAIT_TIMEOUT:
                     raise TimeoutExpired(self.args, timeout)
                 self.returncode = _subprocess.GetExitCodeProcess(self._handle)
             return self.returncode
@@ -1220,8 +1223,10 @@ class Popen(object):
             return (stdout, stderr)
 
         def send_signal(self, sig):
-            """Send a signal to the process
-            """
+            """Send a signal to the process."""
+            # Don't signal a process that we know has already died.
+            if self.returncode is not None:
+                return
             if sig == signal.SIGTERM:
                 self.terminate()
             elif sig == signal.CTRL_C_EVENT:
@@ -1232,8 +1237,10 @@ class Popen(object):
                 raise ValueError("Unsupported signal: %s" % sig)
 
         def terminate(self):
-            """Terminates the process
-            """
+            """Terminates the process."""
+            # Don't terminate a process that we know has already died.
+            if self.returncode is not None:
+                return
             _subprocess.TerminateProcess(self._handle, 1)
 
         kill = terminate
@@ -1878,7 +1885,9 @@ class Popen(object):
         def send_signal(self, sig):
             """Send a signal to the process
             """
-            os.kill(self.pid, sig)
+            # Skip signalling a process that we know has already died.
+            if self.returncode is None:
+                os.kill(self.pid, sig)
 
         def terminate(self):
             """Terminate the process with SIGTERM
